/*

Parser.java
  -- classes implementing an interval arithmetic expression parser
     as part of the "ia_parser library" version 0.1beta1, 10/97

Copyright (C) 1997 Timothy J. Hickey

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

 */





package ia_parser;

import java.lang.*;
import java.io.*;
import ia_math.RealInterval;
/**
 * the Parser class contains methods for reading interval arithmetic
 * expressions and interval arithmetic constraint sets. It returns
 * objects of the "Exp" which can be evaluated, narrowed, etc.
 *
 * @author Tim Hickey
 * @version 0.0 -- 13 June 1997
 */
public class Parser {

  private static String readLine(InputStream inStream) {
    StringBuffer buf = new StringBuffer();
    int c= -1;
    boolean done=false ;


    try {
          System.out.print(">> ");System.out.flush();
          while (!done) {
              c = inStream.read();
              done = ((c=='\n') || (c==-1));
              if (!done) buf.append((char) c);
          }
          return(buf.toString());
        } catch (Exception e) {                        // catch exceptions
          System.out.println(
              "ia_parser.Parser.readLine: \n" +
              "    Found an I/O exception " + e);
          System.exit(0);
        } finally {                                    // close out  
        }

    return(null);
  }


  /**
   * Read a constraint set from the input stream.
   * This returns an <b> Exp</b> object
   * which represents the parse tree of the interval arithmetic
   * constraint set which was read. It reads and attempts
   * to parse the entire string, up to the end of file. 
   */
  public static Exp parse(InputStream inStream) {
    try {
      parser parser_obj = new parser();
      parser_obj.scanner.prog = inStream;
      parser_obj.parse();
      return parser_obj.get_tree();
      } catch (Exception e) {                        // catch exceptions
          System.out.println(
             "ia_parser.Parser.parse(): \n"+
             "Found an exception " + e + "\n");
      } finally {
        }
    return null;
  }

  /**
   * Read a constraint set from a string.
   */
  public static Exp parseString(String string) {
    return parse(new StringBufferInputStream(string));
  }
    
  /**
   * Read a constraint set from a string from the next line of the
   * input stream. 
   */
  public static Exp parseLine(InputStream inStream) {
    String inputLine = readLine(inStream);
    if (inputLine != null)
      return parseString(inputLine);
    else return null;
  }

  /**
   * A simple read/narrow/print loop.
   */
  public static void main(String[] args) {
    RealIntervalTable vars = new RealIntervalTable();

    System.out.println(
    "\n\n\n     Brandeis IAsolver (text-based version 0.1beta1)\n\n\n");

    while (true) {
      int i;
      boolean narrow_result;
      RealInterval eval_result=null;
      Exp tree;

      tree = parseLine(System.in);

      while (tree==null) {
	System.out.println("Please enter an expression or a constraint.\n");
        tree = parseLine(System.in);
      }

        System.out.println(
        "just parsed "+ tree);

        tree.bindVars(vars);

        i = 0; narrow_result = true;
        while ((i < 10) && (narrow_result)) {
          narrow_result = tree.narrow();
	  //          System.out.println(
	  //            "narrowing step i = " + i 
	  //            + vars.toString() + "\n\n");
          i = i + 1; 
	}
        System.out.println(
          "evaluation yields "+ narrow_result
         +"\nnew variable intervals are \n" 
         + vars.toString() + "\n");

        if (!narrow_result) {
          vars = new RealIntervalTable();
          tree.bindVars(vars);
	}
    }
  }
}

